# Counterfactual Sensitivity and Robustness

There are three parts to this code:
1. Replication codes for the Chiappori, Salanie, and Weiss (2017) application
2. Replication codes for the Rust (1987) application
3. General-purpose codes to implement the methodology.

## Software requirements

You will need:

* Julia version 1.6.1 +
* Artelys Knitro version 12.4.0 +

All scripts have been tested in Julia v1.6.4 and Knitro v12.4.0 on macOS X. The code in this repository depends on other packages. Run `import Pkg; Pkg.activate("."); Pkg.instantiate()` in the root directory to install all necessary packages.

## Replication files for the Chiappori, Salanie, and Weiss (2017) application

The folder `examples/CSW` contains the codes for this application.

The file `plot_figures.jl` plots all figures and tables for this application.

To generate results, first run
* `csw_main_1.jl` (computes estimates and bootstrap CSs without exchangeability)
* `csw_main_2.jl` (computes estimates and bootstrap CSs with exchangeability)
* `csw_proj_1.jl` (computes projection CSs without exchangeability)
* `csw_proj_2.jl` (computes projection CSs with exchangeability)
* `csw_phi.jl` (computes estimates with different divergences)
* `csw_inner.jl` (computes estimates holding θ fixed).

As there are 28 cohorts and the estimates and CSs for the upper and lower bounds are computed cohort-wise, each of the above files is designed to be run in parallel on a cluster. The accompanying `.sh` files may be used for this purpose.

To run these `.jl` files on a single computer, comment out the line
```
array_value = Base.parse(Int, ENV["SLURM_ARRAY_TASK_ID"])
```
in each file and replace it with `array_value = x` with `x` an integer between 1 and 56 (or 1 and 28 for `csw_inner.jl`): `array_values` between 1 and 28 compute the lower bounds in each cohort; values between 29 and 56 compute the upper bound in each cohort.

Data for this application is stored in `CSW/data/probs_white.RData`. It is generated by running `Generate_probs_white.R` in R using data from the replication files of Chiappori et al. (2017).

## Replication files for the Rust (1987) application

The folder `examples/CSW` contains the codes for this application.

The file `plot_figures.jl` plots all figures and generates tabulates numerical results for this application.

To generate results, first run
* `rust_main.jl` (computes estimates)
* `rust_proj.jl` (computes projection CSs; must run `rust_main.jl` first)
* `rust_phi.jl` (computes estimates with different divergences)

Each of the above files is designed to be run in parallel on a cluster. The accompanying `.sh` files may be used for this purpose.

To run these `.jl` files on a single computer, comment out the line
```
array_value = Base.parse(Int, ENV["SLURM_ARRAY_TASK_ID"])
```
in each file and replace it with `array_value = 1` (for upper bound) or `= 2` (for lower bound) for `rust_main.jl` and `rust_proj.jl`, and `array_value = x` with `x` integer between 1 and 4 in `rust_phi.jl`. Note that these files require considerable memory to store the Jacobian of the moments across many Monte Carlo draws, which may diminish performance if run on a computer with insufficient memory.

Each of these files loads estimates of CCPs and other model-specific parameters from `rust_initialize.jld`. This file is generated by running `rust_initialize.jl` first.
Data for this application is stored in `Rust/data/a530875.RData`. It is generated by running `rust.R` which converts Rust's `a530875.asc` file into a `.RData` file.

## Guide to General-Purpose Codes

See the README in the `src` folder.
