# Bootstrap match probability estimates using normal approximation
function csw_generate_bootstrap_draws(obj, data, nT, J, C, B)

    Random.seed!(1234567)
    V = zeros(2 * nT, 2 * nT)
    V[1:nT, 1:nT] .= data["varpwomen"][J, C, :, :]
    V[nT + 1:2 * nT, nT + 1:2 * nT] .= data["varpwomen"][J + 1, C, :, :]
    s = Matrix(cholesky(V).L)
    z = zeros(obj.d, B)
    for b in 1:B
        z[2 * (nT + 1) + 1:end, b] .= s * randn(2 * nT)
    end

    V_full = zeros(obj.d, obj.d)
    V_full[2 * (nT + 1) + 1:obj.d, 2 * (nT + 1) + 1:obj.d] .= V

    return z, V_full

end

# Rectangular CS for match probabilities using normal approximation
function confidence_set_P(α, data, nT, J, C, B)

    VF = data["varpwomen"]
    V = zeros(2 * nT, 2 * nT)
    V[1:nT, 1:nT] .= VF[J, C, :, :]
    V[nT + 1:2 * nT, nT + 1:2 * nT] .= VF[J + 1, C, :, :]
    (c, s) = rectangular_confidence_set(V, α)
    P_CS = zeros(nT, 2, 2)
    P_CS[:, :, 1] .= reshape((PF[J:J+1, C, :]')[:] - c * s, (nT, 2))
    P_CS[:, :, 2] .= reshape((PF[J:J+1, C, :]')[:] + c * s, (nT, 2))

    return P_CS

end


# Rectangular CS assuming Gaussianity
function rectangular_confidence_set(V, α)

    l = size(V)[1]
    S = Matrix(cholesky(V).L)
    s = sqrt.(diag(V))

    B = 100000
    z = zeros(l, B)

    Random.seed!(1234567)

    for b in 1:B

        z[:, b] = S * randn(l)
        z[:, b] ./= s

    end

    zz = [maximum(abs.(z[:, b])) for b in 1:B]

    return (quantile(zz, 1 - α), s)

end
