# Codes to compute projection CIs for first implementation (no exchangeability)

# Load packages, etc
import Pkg; Pkg.activate("../..")
using KNITRO, ForwardDiff, Calculus, LinearAlgebra, Distributions, Parameters, Random, JLD, RData
include("../../src/boot.jl")        # Load main programs
include("./aux/csw_boot.jl")        # Load model-specific files and programs

# Load CSW data
data = load("./data/probs_white.RData")
PM = data["pmen"]
PF = data["pwomen"]

# Number of types and cohorts
nT, nC = size(PM)[1:2]

# Education level for counterfactual (an integer between 1 and nT - 1)
J = 3

# Cohort (an integer between 1 and nC)
array_value = Base.parse(Int, ENV["SLURM_ARRAY_TASK_ID"])
C = array_value <= nC ? array_value : array_value - nC

# Finding smallest value?
finding_smallest = array_value <= nC

println([Threads.nthreads(), array_value, J, C, finding_smallest]); flush(stdout)

# Initialize parameters at CSW's estimates
# Rows correspond to female type, columns correspond to male type
T = zeros(nT, nT)
for i in 1:nT
    T[i, :] .= log.(PF[i, C, :] ./ (1.0 - sum(PF[i, C, :])))
end
θ_initial = T[J:J + 1, :][:]

# 95% Rectangular CS for probabilities
P_CS = confidence_set_P(0.05, data, nT, J, C, 10000)

# Set up random draws
M = 50000
U = quantile.(GeneralizedExtremeValue(-Base.MathConstants.eulergamma, 1, 0), rhalton(M, nT + 1, singleseed = 1234567))

# Assemble the ObjectiveBundle
obj = PsiObjectiveBundleExplicit(
    δ = 0.01,
    find_smallest = finding_smallest,
    γ = (C = C, J = J, nT = nT, nC = nC, P_CS, σ2 = Base.MathConstants.pi^2 / 6.0),
    moments! = csw_moments_proj!,
    moments_jacobian! = csw_moments_proj_jacobian_threaded!,
    d = 2 * (nT + 1) + 4 * nT,
    inequality_index = vcat(2 * (nT + 1) + 1:2 * (nT + 1) + 4 * nT),
    complement_index = hcat(vcat(2 * (nT + 1) + 1:2 * (nT + 1) + 2 * nT), vcat(2 * (nT + 1) + 2 * nT + 1:2 * (nT + 1) + 4 * nT)),
    l = 2 * nT,
    U = U,
    outer_loop_opt = "./opt/csw_outer_loop_settings_cluster.opt",
    inner_loop_opt = "./opt/csw_inner_loop_settings_cluster.opt",
    lower_limit = -2e1,
    η_min = 1e-4,
    use_cached_x = true)

θ_lower = -12.0 * ones(obj.l)
θ_upper = +12.0 * ones(obj.l)

δ_grid = 10.0.^vcat(-2.0:0.5:2.0)
κ = zeros(length(δ_grid))
flag = zeros(length(δ_grid))
Θ = zeros(obj.l, length(δ_grid))

θ1 = copy(θ_initial)
for (i, δ) in enumerate(δ_grid)
    println(δ); flush(stdout)
    obj.δ = δ
    (κ_temp, θ1_temp, flag_temp) = outer_loop_multi(obj, θ_lower, θ_upper, θ1, 10, 0.01)
    if i == 1 || (obj.find_smallest && κ_temp < κ[i-1]) || (!obj.find_smallest && κ_temp > κ[i-1])
        global (κ[i], θ1, flag[i]) = (κ_temp, θ1_temp, flag_temp)
        global Θ[:, i] .= θ1
    else
        global (κ[i], θ1, flag[i]) = (κ[i-1], Θ[:, i-1], flag[i-1])
        global Θ[:, i] .= θ1
    end
    save(string("./results/csw_proj_1_",J,"_",array_value,".jld"),"δ_grid", δ_grid, "κ", κ, "Θ", Θ, "flag", flag, "C", C, "J", J, "finding_smallest", finding_smallest)
    iter = 1
    while iter <= 5
        println([δ, iter]); flush(stdout)
        (κ_temp, θ1_temp, flag_temp) = outer_loop_multi(obj, θ_lower, θ_upper, θ1, 10, 0.01)
        if (obj.find_smallest && κ_temp < κ[i]) || (!obj.find_smallest && κ_temp > κ[i])
            global (κ[i], θ1, flag[i]) = (κ_temp, θ1_temp, flag_temp)
            global Θ[:, i] .= θ1
            iter += 1
        else
            iter += 5
        end
        save(string("./results/csw_proj_1_",J,"_",array_value,".jld"),"δ_grid", δ_grid, "κ", κ, "Θ", Θ, "flag", flag, "C", C, "J", J, "finding_smallest", finding_smallest)
    end
end
