# Evaluate convex conjugate in-place
function Psi_χ2!(arg1, arg0)
	@inbounds for i in 1:length(arg0)
		if arg0[i] < -1.0
			arg1[i] = -0.5
		else
			arg1[i] = 0.5 * arg0[i]^2 + arg0[i]
		end
	end
end

# Evaluate its derivative in-place
function dPsi_χ2!(arg1, arg0)
	@inbounds for i in 1:length(arg0)
		if arg0[i] < -1.0
			arg1[i] = 0.0
		else
			arg1[i] = arg0[i] + 1.0
		end
	end
end

# Evaluate its second derivative in-place
function ddPsi_χ2!(arg1, arg0)
	@inbounds for i in 1:length(arg0)
		if arg0[i] < -1.0
			arg1[i] = 0.0
		else
			arg1[i] = 1.0
		end
	end
end

# Evaluate convex conjugate in-place
function Psi_l4!(arg1, arg0)
	@inbounds for i in 1:length(arg0)
		if arg0[i] <= -1.0/3.0
			arg1[i] = -0.25
		else
			arg1[i] = 0.25 * (3.0 * arg0[i] + 1.0)^(4.0 / 3.0) - 0.25
		end
	end
end

# Evaluate its derivative in-place
function dPsi_l4!(arg1, arg0)
	@inbounds for i in 1:length(arg0)
		if arg0[i] <= -1.0/3.0
			arg1[i] = 0.0
		else
			arg1[i] = (3.0 * arg0[i] + 1.0)^(1.0 / 3.0)
		end
	end
end

# Evaluate its second derivative in-place
function ddPsi_l4!(arg1, arg0)
	@inbounds for i in 1:length(arg0)
		if arg0[i] <= -1.0/3.0
			arg1[i] = 0.0
		else
			arg1[i] = (3.0 * arg0[i] + 1.0)^(-2.0 / 3.0)
		end
	end
end
